<?php

defined( 'ABSPATH' ) or exit;

if ( ! class_exists( 'WC_Slack_Events' ) ) {

	/**
	 * Slack events handler.
	 *
	 * @since 1.0.0
	 */
	class WC_Slack_Events {

		/** @var WC_Slack_Events|null */
		protected static ?WC_Slack_Events $instance = null;

		/**
		 * Constructor.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {

			$wrapper = $this->get_notification_wrapper();

			// Event for post being published
			if ( wc_string_to_bool( $wrapper['post_new'] ) ) {
				// was previously hooked to `transition_post_status` but a featured image thumbnail was not available in that context
				add_action( 'wp_after_insert_post', [ $this, 'post_new' ], 10, 4 );
			}

			// Event for new order made
			if ( wc_string_to_bool( $wrapper['order_new'] ) ) {
				// WooCommerce default order created event
				add_action( 'woocommerce_new_order', [ $this, 'new_order' ], 10, 2 );
				// WooCommerce Store API event (typically blocks)
				add_action( 'woocommerce_store_api_checkout_order_processed', [ $this, 'new_order_checkout_block' ], 10, 2 );
				// WooCommerce Point of Sale event
				add_action( 'wc_pos_order_created', [ $this, 'new_order' ], 10, 1 );
			}

			// Event for refund
			if ( wc_string_to_bool( $wrapper['refund'] ) ) {
				add_action( 'woocommerce_order_refunded', [ $this, 'refund' ], 10, 2 );
			}

			// Event for back order made
			if ( wc_string_to_bool( $wrapper['back_order'] ) ) {
				add_action( 'woocommerce_product_on_backorder', [ $this, 'back_order' ], 10, 1 );
			}

			// Event for low on stock
			if ( wc_string_to_bool( $wrapper['low_stock'] ) ) {
				add_action( 'woocommerce_low_stock', [ $this, 'low_stock' ], 10, 1 );
			}

			// Event for out of stock
			if ( wc_string_to_bool( $wrapper['out_stock'] ) ) {
				add_action( 'woocommerce_no_stock', [ $this, 'out_stock' ], 10, 1 );
			}

			// Event for new review
			if ( wc_string_to_bool( $wrapper['new_review'] ) ) {
				add_action( 'comment_post', [ $this, 'new_review' ], 10, 1 );
			}

			// Event for customer created
			if ( wc_string_to_bool( $wrapper['new_customer'] ) ) {
				add_action( 'user_register', [ $this, 'new_customer' ], 10, 2 );
			}
		}

		/**
		 * Start the Class when called
		 *
		 * @since 1.0.0
		 */
		public static function get_instance() {

			// If the single instance hasn't been set, set it now.
			if ( null === self::$instance ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Handles posting to Slack when a new post is published.
		 *
		 * @internal
		 *
		 * @see wp_after_insert_post() this will trigger also for posts inserted via WordPress API
		 * The advantage over other hooks is that additional metadata will be available at this point.
		 *
		 * @since 1.0.0
		 *
		 * @param int|mixed $post_id
		 * @param mixed|WP_Post $post
		 * @param bool|mixed $update
		 * @param mixed|WP_Post|null $post_before
		 * @return void
		 */
		public function post_new( $post_id, $post = null, $update = false, $post_before = null ) {

			/**
			 * Filters the post types that should trigger a notification when published.
			 *
			 * @since 1.0.0
			 *
			 * @param string[] $allowed_post_types by default posts and pages
			 */
			$allowed_post_types = apply_filters( 'wcslack_post_publish_types', [ 'post', 'page' ] );

			if ( ! $post_id || ! $post instanceof WP_Post || ! in_array( $post->post_type, $allowed_post_types, true ) ) {
				return;
			}

			// Only send to posts that are published
			if ( 'publish' !== $post->post_status || ( $post_before instanceof WP_Post && 'publish' === $post_before->post_status ) ) {
				return;
			}

			$wc_slack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();
			$api_key = $wrapper['api_key'];
			$channel = $this->get_notification_channels( 'post_new' );
			$emoji   = $this->get_notification_emoji( 'post_new' );

			/**
			 * Filters the post prefix.
			 *
			 * @since 1.0.0
			 *
			 * @param string $prefix default "New Post:"
			 */
			$prefix = (string) apply_filters( 'wcslack_new_post_prefix', __( 'New Post:', 'woocommerce-slack' ) );

			/**
			 * Filters the author name.
			 *
			 * @since 1.0.0
			 *
			 * @param string $author_name escaped
			 */
			$author_data = get_userdata( $post->post_author );
			$author_name = (string) apply_filters( 'wcslack_new_post_author', esc_attr( $author_data->display_name ) );

			/**
			 * Filters the post title.
			 *
			 * We use {@see get_post_field()} to stop automatic {@see get_the_title()} encoding.
			 *
			 * @since 1.0.0
			 *
			 * @param string $title
			 */
			$title = (string) apply_filters( 'wcslack_new_post_title', get_post_field( 'post_title', $post_id ) );

			/**
			 * Filters the post content.
			 *
			 * We use {@see get_post_field()} as {@see get_the_content()} won't work here (we need to feed the post ID).
			 *
			 * @since 1.0.0
			 *
			 * @param string $post_content
			 */
			$post_content = (string) apply_filters( 'wcslack_new_post_content', wp_strip_all_tags( get_post_field( 'post_content', $post_id ) ) );

			/**
			 * Filters the post URL.
			 *
			 * @since 1.0.0
			 *
			 * @param string $url escaped
			 */
			$url = (string) apply_filters( 'wcslack_new_post_url', esc_url( get_permalink( $post_id ) ) );

			if ( $wrapper['post_new_message'] ) {
				$message_raw = $wrapper['post_new_message'];
				$tags        = [ '{post_title}', '{post_content}', '{post_author}', '{post_link}', '[[', ']]' ];
				$replace     = [ $title, $post_content, $author_name, $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $title . __( ' by ', 'woocommerce-slack' ) . $author_name;
				$message .= ' - <' . $url . '|' . __( 'View Post', 'woocommerce-slack' ) . '>';
			}

			/**
			 * Filters the message.
			 *
			 * @since 1.0.0
			 *
			 * @param string $message
			 */
			$message = (string) apply_filters( 'wcslack_new_post_message', $message );

			/**
			 * Filters the legacy attachment fields.
			 *
			 * @since 1.0.0
			 *
			 * @link https://api.slack.com/reference/messaging/attachments#example
			 *
			 * @param array<int, array<string, mixed> $attachment_fields
			 */
			$attachment_fields = (array) apply_filters( 'wcslack_new_post_attachment', [
				[
					'title' => __( 'Post', 'woocommerce-slack' ),
					'value' => '<' . $url . '|' . $title . '>',
					'short' => true,
				],
				[
					'title' => __( 'Author', 'woocommerce-slack' ),
					'value' => $author_name,
					'short' => true,
				],
				[
					'title' => __( 'Post Content', 'woocommerce-slack' ),
					'value' => $post_content,
					'short' => false,
				],
			] );

			$attachment = '';

			// Only give $attachment content if the setting for 'extended notification' is on
			if ( wc_string_to_bool( $wrapper['post_new_extend'] ) ) {

				$attachment = $wc_slack_api->attachment( $attachment_fields );

				// Add any thumbnail, if present
				if ( $thumbnail = get_the_post_thumbnail_url( $post_id ) ) {
					$attachment[0]['image_url'] = $thumbnail;
				}

				// published date as footer timestamp
				$attachment[0]['ts'] = strtotime( $post->post_date );
			}

			$this->log( $message, $channel, $emoji );

			$wc_slack_api->send_message( $api_key, $channel, $emoji, $message, $attachment );
		}

		/**
		 * New order made.
		 *
		 * @since 1.0.0
		 * @internal
		 *
		 * @param int|mixed $order_id
		 * @param mixed|WC_Order|null $maybe_order
		 * @return void
		 */
		public function new_order( $order_id, $maybe_order = null ) {

			if ( ! is_numeric( $order_id ) ) {
				return;
			}

			$order = is_a( $maybe_order, 'WC_Order' ) ? $maybe_order : wc_get_order( $order_id );

			if ( ! $order instanceof WC_Order ) {
				return;
			}

			// Set up Settings Wrapper
			$wcslack_api = new WC_Slack_API();
			$wrapper     = $this->get_notification_wrapper();

			// Settings
			$api_key    = $wrapper['api_key'];
			$free_order = $wrapper['order_new_free'];

			// Channel & Emoji
			$channel = $this->get_notification_channels( 'order_new' );
			$emoji   = $this->get_notification_emoji( 'order_new' );

			// Order Items
			$order_items = $this->prepare_order_item_titles( $order->get_items() );

			// Order Total
			$order_total = $this->format_price_for_notification( $order->get_total() );

			// Order Status
			$order_status = $order->get_status();

			// Order Status Name
			$order_status_name = wc_get_order_status_name( $order_status );

			// Order Attachment Line Items
			$order_line_items = $this->prepare_order_line_items( $order );

			// Order Attachment Shipping
			$order_shipping = $this->prepare_shipping_total( $order );

			// Order Attachment Total
			$order_attach_total = $this->prepare_order_total( $order );

			// Order URL
			$url = $this->get_edit_order_link( (int) $order_id );

			// Send notifications if order total is greater than 0 or if free order notification is enabled
			if ( 0 < $order->get_total() || ( wc_string_to_bool( $free_order ) || ( $order->is_paid() && current_action() !== 'wc_pos_order_created' ) ) ) {

				// Do not send a notification if order status is checkout-draft, this comes from the checkout block experience and results in duplicate notifications
				if ( 'checkout-draft' !== $order_status ) {

					$first_name = $order->get_billing_first_name();
					$last_name  = $order->get_billing_last_name();

					if ( $wrapper['order_new_message'] ) {

						$message_raw = $wrapper['order_new_message'];
						$tags        = [ '{order_id}', '{order_number}', '{first_name}', '{last_name}', '{order_items}', '{order_total}', '{order_link}', '{order_status}', '[[', ']]' ];
						$replace     = [ $order->get_id(), $order->get_order_number(), $first_name, $last_name, $order_items, $order_total, $url, $order_status_name, '<', '>' ];
						$message     = str_replace( $tags, $replace, $message_raw );

					} else {

						/* translators: Placeholder: %s - Order number */
						$prefix = sprintf( __( 'New Order #%s:', 'woocommerce-slack' ), $order->get_order_number() );

						/**
						 * Filters the prefix for the new order message.
						 *
						 * @since 1.0.0
						 *
						 * @param string $prefix
						 */
						$prefix = (string) apply_filters( 'wcslack_new_order_prefix', $prefix );

						$message  = '*' . $prefix . '* ' . $first_name . ' ' . $last_name . __( ' ordered ', 'woocommerce-slack' );
						$message .= $order_items . __( ' for ', 'woocommerce-slack' ) . $order_total;
						$message .= ' - <' . $url . '|' . __( 'View Order', 'woocommerce-slack' ) . '>';
					}

					/**
					 * Filters the new order message.
					 *
					 * @since 1.0.0
					 *
					 * @param string $message
					 * @param int $order_id
					 */
					$message = (string) apply_filters( 'wcslack_new_order_message', $message, $order_id );

					// Only give $attachment content if the setting for 'extended notification' is on
					if ( wc_string_to_bool( $wrapper['order_new_extend'] ) ) {

						/**
						 * Filters the attachment fields.
						 *
						 * @since 1.0.0
						 *
						 * @param array<int, array<string, mixed> $attachment_fields
						 */
						$fields = apply_filters( 'wcslack_new_order_attachment', [
							0 => [
								'title' => __( 'Order', 'woocommerce-slack' ),
								'value' => '<' . $url . '|#' . $order_id . '>',
								'short' => true,
							],
							1 => [
								'title' => __( 'Order Status', 'woocommerce-slack' ),
								'value' => $order_status_name,
								'short' => true,
							],
							2 => [
								'title' => __( 'Customer', 'woocommerce-slack' ),
								'value' => $first_name . ' ' . $last_name,
								'short' => false,
							],
							3 => [
								'title' => __( 'Order Items', 'woocommerce-slack' ),
								'short' => false,
							],
						], $order_id );

						$fields = array_merge( (array) $fields, (array) $order_line_items, (array) $order_shipping, (array) $order_attach_total );

						$attachment = $wcslack_api->attachment( $fields );
					} else {
						$attachment = '';
					}

					$this->log( $message, $channel, $emoji );

					$wcslack_api->send_message( $api_key, $channel, $emoji, $message, $attachment );
				}
			}
		}

		/**
		 * New order method processed by the Checkout Block/Store API.
		 *
		 * @since 1.4.0
		 * @internal
		 *
		 * @param mixed|WC_Order $order order object
		 */
		public function new_order_checkout_block( $order ) : void {

			if ( ! is_a( $order, 'WC_Order' ) ) {
				return;
			}

			$this->new_order( $order->get_id(), $order );
		}

		/**
		 * Refund processed.
		 *
		 * @since 1.5.0
		 * @internal
		 *
		 * @param int|mixed $order_id
		 * @param int|mixed $refund_id
		 * @return void
		 */
		public function refund( $order_id, $refund_id ) : void {

			if ( ! is_numeric( $order_id ) || ! is_numeric( $refund_id ) ) {
				return;
			}

			$order = wc_get_order( $order_id );

			if ( ! $order instanceof WC_Order ) {
				return;
			}

			$refund = wc_get_order( $refund_id );

			if ( ! $refund instanceof WC_Order_Refund ) {
				return;
			}

			// Set up Settings Wrapper
			$wcslack_api = new WC_Slack_API();
			$wrapper     = $this->get_notification_wrapper();

			// Settings
			$api_key = $wrapper['api_key'];

			// Channel & Emoji
			$channel = $this->get_notification_channels( 'refund' );
			$emoji   = $this->get_notification_emoji( 'refund' );

			// Refunded items titles
			$refunded_items = $this->prepare_order_item_titles( $refund->get_items() );

			// Order items titles
			$order_items = $this->prepare_order_item_titles( $order->get_items() );

			// Refund total
			$refund_total = $this->format_price_for_notification( abs( $refund->get_total() ) );

			// Refunded items
			$refunded_line_items = $this->prepare_order_line_items( $refund );

			// Refund attachment total
			$order_attach_total = $this->prepare_order_total( $order );

			// Refund reason
			$refund_reason = $refund->get_reason() ?: _x( 'No reason given', 'Refund reason', 'woocommerce-slack' );

			// Order URL
			$url = $this->get_edit_order_link( (int) $order_id );

			if ( $wrapper['refund_message'] ) {

				$message_raw = $wrapper['refund_message'];
				$tags        = [ '{order_id}', '{order_number}', '{refund_reason}', '{refunded_items}', '{order_items}', '{refunded_total}', '{order_link}', '[[', ']]' ];
				$replace     = [ $order->get_id(), $order->get_order_number(), $refund_reason, $refunded_items, $order_items, $refund_total, $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );

			} else {

				/* translators: Placeholder: %s - Order number */
				$prefix = sprintf( __( 'Refund for Order #%s:', 'woocommerce-slack' ), $order->get_order_number() );

				/**
				 * Filters the prefix for the new order message.
				 *
				 * @since 1.5.0
				 *
				 * @param string $prefix
				 */
				$prefix = (string) apply_filters( 'wcslack_new_order_prefix', $prefix );

				$message = '*' . $prefix . '* ';
				$message .= $refunded_items . __( ' for ', 'woocommerce-slack' ) . $refund_total;
				$message .= ' - <' . $url . '|' . __( 'View Order', 'woocommerce-slack' ) . '>';
			}

			/**
			 * Filters the refund message.
			 *
			 * @since 1.5.0
			 *
			 * @param string $message
			 * @param int $order_id
			 */
			$message = (string) apply_filters( 'wcslack_refund_message', $message, $order_id );

			/**
			 * Filters the attachment fields.
			 *
			 * @since 1.5.0
			 *
			 * @param int $refund_id
			 * @param array<int, array<string, mixed> $attachment_fields
			 */
			$fields = (array) apply_filters( 'wcslack_refund_attachment', [
				0 => [
					'title' => __( 'Order', 'woocommerce-slack' ),
					'value' => '<' . $url . '|#' . $order_id . '>',
					'short' => true,
				],
				1 => [
					'title' => __( 'Refund Reason', 'woocommerce-slack' ),
					'value' => $refund_reason,
					'short' => true,
				],
				2 => [
					'title' => __( 'Refunded Items', 'woocommerce-slack' ),
					'short' => false,
				],
			], $refund_id );

			// Only give $attachment content if the setting for 'extended notification' is on
			if ( wc_string_to_bool( $wrapper['refund_extend'] ) ) {
				$attachment = $wcslack_api->attachment( array_merge( $fields, $refunded_line_items, $order_attach_total ) );
			} else {
				$attachment = '';
			}

			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message, $attachment );
		}

		/**
		 * Back order placed.
		 *
		 * @since 1.0.0
		 * @internal
		 *
		 * @param array<string,mixed>|mixed $args
		 * @return void
		 */
		public function back_order( $args ) : void {

			$wcslack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();

			$api_key = $wrapper['api_key'];

			$channel = $this->get_notification_channels( 'back_order' );
			$emoji   = $this->get_notification_emoji( 'back_order' );

			// Back Order Prefix
			$prefix = apply_filters( 'wcslack_back_order_prefix', __( 'Back Order:', 'woocommerce-slack' ) );

			// Product on Backorder
			$product           = $args['product'];
			$product_backorder = sprintf( __( 'Product (#%1$d %2$s) is on backorder.', 'woocommerce-slack' ), $product->get_id(), $product->get_title() );

			// Product URL
			$url = admin_url( 'post.php?post=' . $product->get_id() . '&action=edit' );

			// Back Order Message
			if ( $wrapper['back_order_message'] ) {
				$message_raw = $wrapper['back_order_message'];
				$tags        = [ '{product_name}', '{product_id}', '{product_link}', '[[', ']]' ];
				$replace     = [ $product->get_title(), $product->get_id(), $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $product_backorder;
				$message .= ' - <' . $url . '|' . __( 'View Product', 'woocommerce-slack' ) . '>';
			}

			// Filter for the Message
			$message = apply_filters( 'wcslack_back_order_message', $message );

			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message );
		}

		/**
		 * Item Low on Stock
		 *
		 * @since 1.0.0
		 */
		public function low_stock( $product ) {

			$wcslack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();

			$api_key = $wrapper['api_key'];

			$channel = $this->get_notification_channels( 'low_stock' );
			$emoji   = $this->get_notification_emoji( 'low_stock' );

			// Low on Stock Prefix
			$prefix = apply_filters( 'wcslack_low_stock_prefix', __( 'Low on Stock:', 'woocommerce-slack' ) );

			// Product Low on Stock
			$product_low_stock = sprintf( __( 'Product (#%1$d %2$s) is low on stock.', 'woocommerce-slack' ), $product->get_id(), $product->get_title() );

			// Product URL
			$url = admin_url( 'post.php?post=' . $product->get_id() . '&action=edit' );

			// Low on Stock Message
			if ( $wrapper['low_stock_message'] ) {
				$message_raw = $wrapper['low_stock_message'];
				$tags        = [ '{product_name}', '{product_id}', '{product_link}', '[[', ']]' ];
				$replace     = [ $product->get_title(), $product->get_id(), $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $product_low_stock;
				$message .= ' - <' . $url . '|' . __( 'View Product', 'woocommerce-slack' ) . '>';
			}

			// Filter for the Message
			$message = apply_filters( 'wcslack_low_stock_message', $message );

			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message );
		}

		/**
		 * Item Out of Stock
		 *
		 * @since 1.0.0
		 */
		public function out_stock( $product ) {

			$wcslack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();

			$api_key = $wrapper['api_key'];

			$channel = $this->get_notification_channels( 'out_stock' );
			$emoji   = $this->get_notification_emoji( 'out_stock' );

			// Out of Stock Prefix
			$prefix = apply_filters( 'wcslack_out_stock_prefix', __( 'Out of Stock:', 'woocommerce-slack' ) );

			// Product Out of Stock
			$product_out_stock = sprintf( __( 'Product (#%1$d %2$s) is out of stock.', 'woocommerce-slack' ), $product->get_id(), $product->get_title() );

			// Product URL
			$url = admin_url( 'post.php?post=' . $product->get_id() . '&action=edit' );

			// Out of Stock Message
			if ( $wrapper['out_stock_message'] ) {
				$message_raw = $wrapper['out_stock_message'];
				$tags        = [ '{product_name}', '{product_id}', '{product_link}', '[[', ']]' ];
				$replace     = [ $product->get_title(), $product->get_id(), $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $product_out_stock;
				$message .= ' - <' . $url . '|' . __( 'View Product', 'woocommerce-slack' ) . '>';
			}

			// Filter for the Message
			$message = apply_filters( 'wcslack_out_stock_message', $message );

			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message );
		}

		/**
		 * New Review on Product
		 *
		 * @since 1.0.0
		 */
		public function new_review( $comment_id ) {

			$wcslack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();

			$api_key = $wrapper['api_key'];

			$channel = $this->get_notification_channels( 'new_review' );
			$emoji   = $this->get_notification_emoji( 'new_review' );

			// Get Comment Data bsed on the ID
			$comment_data = get_comment( $comment_id );

			// We really don't want to send notifications about SPAM
			if ( 'spam' === $comment_data->comment_approved ) {
				return;
			}

			// Get WP_Post Class based on the review
			$product_data = get_post( $comment_data->comment_post_ID );

			// We only want comments that are on products, ie. reviews
			if ( 'product' !== $product_data->post_type ) {
				return;
			}

			// Reviewer Name
			$reviewer = apply_filters( 'wcslack_new_review_reviewer_name', $comment_data->comment_author );

			// Product Name
			$product_name = apply_filters( 'wcslack_new_review_product_name', $product_data->post_title );

			// Review Rating
			$rating         = get_comment_meta( $comment_id, 'rating', true );
			$rating_star    = apply_filters( 'wcslack_new_review_rating_star', ':star:' );
			$display_rating = str_repeat( $rating_star, $rating );

			// Review Content
			$review_content = apply_filters( 'wcslack_new_review_content', $comment_data->comment_content );

			// New Review Prefix
			$prefix = apply_filters( 'wcslack_new_review_prefix', __( 'New Review:', 'woocommerce-slack' ) );

			// Review URL
			$url = apply_filters( 'wcslack_new_review_url', get_comment_link( $comment_data ) );

			// New Review Message
			if ( $wrapper['new_review_message'] ) {
				$message_raw = $wrapper['new_review_message'];
				$tags        = [ '{reviewer_name}', '{product_name}', '{review_content}', '{review_rating}', '{review_link}', '[[', ']]' ];
				$replace     = [ $reviewer, $product_name, $review_content, $display_rating, $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $reviewer . __( ' on ', 'woocommerce-slack' ) . $product_name;
				$message .= ' ' . $display_rating;
				$message .= ' - <' . $url . '|' . __( 'View Review', 'woocommerce-slack' ) . '>';
			}

			// Filter for the Message
			$message = apply_filters( 'wcslack_new_review_message', $message );

			// Attachment
			$fields = apply_filters( 'wcslack_new_review_attachment', [
				0 => [
					'title' => __( 'Reviewer', 'woocommerce-slack' ),
					'value' => $reviewer,
					'short' => true,
				],
				1 => [
					'title' => __( 'Rating', 'woocommerce-slack' ),
					'value' => $display_rating,
					'short' => true,
				],
				2 => [
					'title' => __( 'Product', 'woocommerce-slack' ),
					'value' => '<' . esc_url( get_permalink( $product_data->ID ) ) . '|' . $product_name . '>',
					'short' => false,
				],
				3 => [
					'title' => __( 'Review', 'woocommerce-slack' ),
					'value' => $review_content,
					'short' => false,
				],
			] );

			// Only give $attachment content if the setting for 'extended notification' is on
			if ( wc_string_to_bool( $wrapper['new_review_extend'] ) ) {
				$attachment = $wcslack_api->attachment( $fields );
			} else {
				$attachment = '';
			}

			// Write message contents to log
			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message, $attachment );
		}

		/**
		 * New customer created.
		 *
		 * @since 1.0.0
		 * @since 1.3.0 The third parameter is deprecated.
		 *
		 * @param int $customer_id customer ID
		 * @param array $customer_data the raw array of data passed to wp_insert_user()
		 * @param bool $deprecated deprecated argument
		 */
		public function new_customer( $customer_id, $customer_data, $deprecated = false ) {

			if ( 'customer' !== $customer_data['role'] ) {
				return;
			}

			$wcslack_api = new WC_Slack_API();

			$wrapper = $this->get_notification_wrapper();

			$api_key = $wrapper['api_key'];

			$channel = $this->get_notification_channels( 'new_customer' );
			$emoji   = $this->get_notification_emoji( 'new_customer' );

			// User Data by Customer ID.
			$user_data = get_user_by( 'id', $customer_id );

			/**
			 * Filters the display name of the new customer.
			 *
			 * @since 1.0.0
			 *
			 * @param string $user_name user's display name
			 */
			$user_name = apply_filters( 'wcslack_new_customer_name', $user_data->display_name );

			/**
			 * Filters the new customer prefix.
			 *
			 * @since 1.0.0
			 *
			 * @param string $prefix new customer prefix
			 */
			$prefix = apply_filters( 'wcslack_new_customer_prefix', __( 'New Customer:', 'woocommerce-slack' ) );

			/**
			 * Filters the new customer URL.
			 *
			 * @since 1.0.0
			 *
			 * @param string $url new customer URL
			 */
			$url = apply_filters( 'wcslack_new_customer_url', esc_url( admin_url( 'user-edit.php?user_id=' . $customer_id ) ) );

			// New Customer Message.
			if ( $wrapper['new_customer_message'] ) {
				$message_raw = $wrapper['new_customer_message'];
				$tags        = [ '{customer_name}', '{customer_link}', '[[', ']]' ];
				$replace     = [ $user_name, $url, '<', '>' ];
				$message     = str_replace( $tags, $replace, $message_raw );
			} else {
				$message = '*' . $prefix . '* ' . $user_name . __( ' Registered', 'woocommerce-slack' );
				$message .= ' - <' . $url . '|' . __( 'View Customer', 'woocommerce-slack' ) . '>';
			}

			/**
			 * Filters the new customer message.
			 *
			 * @since 1.0.0
			 *
			 * @param array|string $message new customer message
			 */
			$message = apply_filters( 'wcslack_new_customer_message', $message );

			$this->log( $message, $channel, $emoji );

			$wcslack_api->send_message( $api_key, $channel, $emoji, $message );
		}

		/**
		 * Returns the notification wrapper data.
		 *
		 * @since 1.0.0
		 *
		 * @return array<string, mixed>
		 */
		private function get_notification_wrapper() : array {

			return ( new WC_Slack_Settings() )->wrapper();
		}

		/**
		 * Get channel to Send Notification too (based on settings).
		 *
		 * @since 1.0.0
		 *
		 * @param string $event
		 * @return string|string[]
		 */
		private function get_notification_channels( string $event ) {

			$settings = new WC_Slack_Settings();
			$wrapper  = $settings->wrapper();

			$channel_default = $wrapper['channel'];
			$channel_custom  = $wrapper[ $event . '_channel' ];

			if ( ! empty( $channel_custom ) && ( 'select' !== $channel_custom ) ) {
				return $channel_custom;
			} else {
				return $channel_default;
			}
		}

		/**
		 * Get Emoji for Notification (based on settings).
		 *
		 * @since 1.0.0
		 *
		 * @param string $event
		 * @return string
		 */
		private function get_notification_emoji( string $event ) : string {

			$settings = new WC_Slack_Settings();
			$wrapper  = $settings->wrapper();

			$emoji_default = $wrapper['emoji'];
			$emoji_custom  = $wrapper[ $event . '_emoji' ];

			if ( ! empty( $emoji_custom ) ) {
				return is_string( $emoji_custom ) ? $emoji_custom : (string) $emoji_default;
			} else {
				return is_string( $emoji_default ) ? $emoji_default : '';
			}
		}

		/**
		 * Gets the order line items.
		 *
		 * @since 1.0.0
		 *
		 * @param int|WC_Order|WC_Order_Refund $order order object or ID
		 * @return array<string, mixed>
		 */
		private function prepare_order_line_items( $order ) : array {

			$order       = $order instanceof WC_Order || $order instanceof WC_Order_Refund ? $order : wc_get_order( $order );
			$order_items = $order ? $order->get_items() : [];
			$line_items  = [];

			foreach ( $order_items as $item ) {

				$item_name     = sprintf( '<%s|%s>', get_permalink( $item['product_id'] ), $item['name'] );
				$item_quantity = 'x ' . abs( $item['qty'] );
				$item_total    = $this->format_price_for_notification( abs( (float) $order->get_line_total( $item, true ) ) );

				$line_items[] = [
					'value' => $item_name . ' ' . $item_quantity . ' - ' . $item_total,
					'short' => false,
				];
			}

			return $line_items;
		}

		/**
		 * Prepares the order shipping data.
		 *
		 * @since 1.0.0
		 *
		 * @param int|WC_Order $the_order order object or ID
		 * @return array<string, mixed>
		 */
		private function prepare_shipping_total( $the_order ) : array {

			$order = $the_order instanceof WC_Order ? $the_order : wc_get_order( $the_order );

			$shipping_data  = [];
			$shipping_total = $order ? (float) $order->get_shipping_total() : null;

			if ( is_numeric( $shipping_total ) ) {
				$shipping_data[] = [
					'title' => __( 'Shipping', 'woocommerce-slack' ) . ' - ' . $this->format_price_for_notification( abs( $shipping_total ) ),
					'short' => false,
				];
			}

			return $shipping_data;
		}

		/**
		 * Prepares the order total data.
		 *
		 * @since 1.0.0
		 *
		 * @param int|WC_Order $the_order order object or ID
		 * @return array<string, mixed>
		 */
		private function prepare_order_total( $the_order ) : array {

			$order = $the_order instanceof WC_Order ? $the_order : wc_get_order( $the_order );

			$totals_data = [];
			$order_total = $order ? (float) $order->get_total() : null;

			if ( is_numeric( $order_total ) ) {
				$totals_data[] = [
					'title' => __( 'Order Total', 'woocommerce-slack' ) . ' - ' . $this->format_price_for_notification( abs( $order_total ) ),
					'short' => false,
				];
			}

			return $totals_data;
		}

		/**
		 * Prepare order items titles for use in notifications
		 *
		 * @since 1.1.4
		 *
		 * @param array<string, mixed>|mixed $order_items assuming {@see WC_Order::get_items()}
		 * @return string
		 */
		private function prepare_order_item_titles( $order_items ) : string {

			if ( ! is_array( $order_items ) ) {
				return '';
			}

			$order_items_array = [];

			foreach ( $order_items as $item ) {
				$order_items_array[] = $item->get_name();
			}

			$order_item_titles = implode( ', ', $order_items_array );

			unset( $order_items_array );

			return $order_item_titles;
		}

		/**
		 * Formats the price with the current currency.
		 *
		 * This ensures that:
		 * - the currency symbol is placed in the expected position
		 * - the currency symbol entity is decoded, e.g. `&euro;` to `€` or `&#36;` to `$`
		 * - the price is stripped of any HTML tags
		 * - the price is formatted as a string with the correct number of decimal places and separators
		 *
		 * @since 1.5.0
		 *
		 * @param float|int|string $price
		 * @return string
		 */
		private function format_price_for_notification( $price ) : string {

			return html_entity_decode( wp_strip_all_tags( wc_price( $price ) ) );
		}

		/**
		 * Gets the order edit order link for an order given its ID.
		 *
		 * @since 1.5.0
		 *
		 * @param int $order_id
		 * @return string URL
		 */
		private function get_edit_order_link( int $order_id ) : string {

			// accounts for HPOS
			if ( class_exists( Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController::class ) && wc_get_container()->get( Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController::class )->custom_orders_table_usage_is_enabled() ) {
				$url = admin_url( 'admin.php?page=wc-orders&action=edit&id=' . $order_id );
			} else {
				$url = admin_url( 'post.php?post=' . $order_id . '&action=edit' );
			}

			return $url;
		}

		/**
		 * Logs the message, channel(s), and emoji to the debug log.
		 *
		 * @since 1.0.0
		 *
		 * @param string $message
		 * @param string|string[] $channel
		 * @param string $emoji
		 * @return void
		 */
		private function log( string $message, $channel, string $emoji ) : void {

			KoiLab\WC_Slack\Utilities\Log_Utils::debug( // phpcs:ignore
				print_r( [
					'Channels' => $channel,
					'Emoji'    => $emoji,
					'Message'  => $message,
				], true ) );
		}

	}

}
