<?php
/**
 * Slack for WooCommerce Events.
 *
 * @package WC_Slack
 * @since   1.1.0
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'WC_Slack_API' ) ) {
	/**
	 * WC Slack Class:
	 */
	class WC_Slack_API {

		protected static $instance = null;

		public function __construct() {}

		/**
		 * Start the Class when called
		 *
		 * @since 1.0.0
		 */
		public static function get_instance() {

			// If the single instance hasn't been set, set it now.
			if ( null === self::$instance ) {
				self::$instance = new self();
			}

			return self::$instance;
		}


		/**
		 * List all channels.
		 *
		 * @since 1.0.0
		 */
		public function all_channels( $api_key ) {
			$channels_array = [];

			if ( ! empty( get_transient( 'wcslack_429_status' ) ) ) {
				return $channels_array;
			}

			$url = add_query_arg(
				[
					'types'            => 'public_channel,private_channel',
					'exclude_archived' => 'true',
					'limit'            => '1000',
				],
				'https://slack.com/api/conversations.list'
			);

			$resp = wp_remote_get(
				$url,
				[
					'user-agent' => get_bloginfo( 'name' ) . ' / 1.0',
					'headers'    => [
						'Content-Type'  => 'application/json',
						'Authorization' => 'Bearer ' . $api_key,
					],
				]
			);

			if ( 429 === wp_remote_retrieve_response_code( $resp ) ) {
				set_transient( 'wcslack_429_status', $resp, MINUTE_IN_SECONDS );
				return $channels_array;
			}

			$body    = wp_remote_retrieve_body( $resp );
			$decoded = json_decode( $body, true );

			if ( empty( $decoded['channels'] ) ) {
				return $channels_array;
			}

			$channels = $decoded['channels'];

			foreach ( $channels as $channel ) {

				$channels_array[ $channel['id'] ] = '#' . $channel['name'];

			}

			return $channels_array;
		}

		/**
		 * Sends a Message / Notification to Slack.
		 *
		 * @since 1.0.0
		 *
		 * @param string|null $api_key Slack API key
		 * @param string[]|string|null $channels one or more channels
		 * @param string $emoji emoji to use next to the message
		 * @param string $message the message
		 * @param array|string|null $attachments optional attachments or fields
		 * @return void
		 */
		public function send_message( ?string $api_key, $channels, string $emoji, string $message, $attachments = '' ) :  void {

			$wrapper = $this->wrapper();

			/**
			 * Sets the Slack user to site name unless specified (filterable).
			 *
			 * @since 1.0.0
			 *
			 * @param string $user Slack user to appear posting the message
			 */
			$user = apply_filters( 'wcslack_message_username', $wrapper['name'] ?? get_bloginfo( 'name' ) );

			// Slack API URL to send message & query arg fields for it
			$fields = [
				'token'      => $api_key,
				'username'   => htmlspecialchars_decode( $user ),
				'text'       => $message,
				'icon_emoji' => $emoji,
			];

			if ( $attachments ) {
				$fields['attachments'] = $attachments;
			}

			/**
			 * Send the message.
			 *
			 * If channels is an array, we will need to send to each of them through separate API calls for each channel.
			 */
			if ( is_array( $channels ) ) {

				foreach ( $channels as $channel ) {

					$fields['channel'] = rawurlencode( $channel );

					$this->send( $fields );
				}
			} else {

				$fields['channel'] = rawurlencode( $channels );

				$this->send( $fields );
			}
		}

		/**
		 * Helper method for sending the actual message  to chat.postMessage Slack API.
		 *
		 * @param array $fields
		 * @return WP_Error|array
		 */
		public function send( array $fields = [] ) {

			$error   = false;
			$payload = wp_json_encode( $fields );
			$url     = 'https://slack.com/api/chat.postMessage';

			// The token needs to be passed in a header and cannot be part of the JSON-encoded body.
			$token = empty( $fields['token'] ) ? get_option( 'wc_slack_access_token' ) : $fields['token'];

			unset( $fields['token'] );

			$response = wp_remote_post(
				$url,
				[
					'user-agent' => get_bloginfo( 'name' ) . ' / 1.0',
					'headers'    => [
						'Content-Type'  => 'application/json',
						'Authorization' => 'Bearer ' . $token,
					],
					'body'       => $payload,
				]
			);

			if ( is_wp_error( $response ) ) {
				$body  = $response->get_error_message();
				$code  = $response->get_error_code();
				$error = true;
			} else {
				$body    = wp_remote_retrieve_body( $response );
				$code    = wp_remote_retrieve_response_code( $response );
				$decoded = (array) json_decode( $body, true );
			}

			if ( $error || empty( $decoded['ok'] ) ) {
				\KoiLab\WC_Slack\Utilities\Log_Utils::log( "Unexpected response from {$url} \nStatus: {$code} \nRequest: {$payload} \nResponse: {$body}", WC_Log_Levels::ERROR );
			}

			return $response;
		}


		/**
		 * Returns a normalized array of data for legacy Slack attachments.
		 *
		 * @since 1.0.0
		 *
		 * @param array $fields
		 * @param string|null $title
		 * @param string|null $summary
		 * @return array<string, mixed>
		 */
		public function attachment( array $fields = [], ?string $title = '', ?string $summary = '' ) : array {

			return [
				[
					'fallback' => $summary,
					'title'    => $title ?: '',
					'text'     => $summary ?: '',
					'color'    => 'good',
					'fields'   => $fields,
				],
			];
		}


		/**
		 * Checks if submitted API Key is valid
		 *
		 * @since 1.0.0
		 *
		 * @return HTTP Status (true / false / ok etc. )
		 */
		public function valid( $api_key ) {

			$resp = wp_remote_post(
				'https://slack.com/api/auth.test',
				[
					'user-agent' => get_bloginfo( 'name' ) . ' / 1.0',
					'headers'    => [
						'Content-Type'  => 'application/json',
						'Authorization' => 'Bearer ' . $api_key,
					],
				]
			);

			if ( is_wp_error( $resp ) ) {

				return $resp;

			} else {

				$body    = wp_remote_retrieve_body( $resp );
				$decoded = (array) json_decode( $body, true );

				if ( array_key_exists( 'ok', $decoded ) ) {
					$status = $decoded['ok'];
				}

				return $status;

			}
		}

		/**
		 * Settings Wrapper
		 *
		 * @since 1.1.0
		 */
		public function wrapper() {
			$WC_Slack_Settings = new WC_Slack_Settings();

			return $WC_Slack_Settings->wrapper();
		}
	}

}
